@@ -7,18 +7,22 @@ using namespace RcppParallel;
 
 //==============================================================================
 
-//-----------------------------------------------------------
+//-------------------------------------------------------------
 // return JSSA probability animal n detected at least once   
 // Pledger et al. 2010 Eqn (3) (mixtures outside)            
-//-----------------------------------------------------------
+// 2020-10-25 changed 'pm' to 'alpha' so consistent with allhistsecrparallelcpp
+// 2020-10-28 new algorithm for movement models
+// See overlapping code in getpdotbd() of allhistsecrparallelcpp()
+// also R-only version in PCH1.R
+//-------------------------------------------------------------
 
 // [[Rcpp::export]]
 NumericVector PCH1cpp (int type, int x, int nc, int jj, 
-                       const IntegerVector cumss, int nmix,
-                       const NumericMatrix openval0, 
-                       const IntegerVector PIA0,  
-                       const IntegerVector PIAJ,  
-                       const NumericVector intervals)
+    const IntegerVector cumss, int nmix,
+    const NumericMatrix openval0, 
+    const IntegerVector PIA0,  
+    const IntegerVector PIAJ,  
+    const NumericVector intervals)
 {
     RVector<int> cumssR(cumss);
     RMatrix<double> openval0R(openval0);
@@ -66,57 +70,57 @@ NumericVector PCH1cpp (int type, int x, int nc, int jj,
 // probability of not being detected in primary session j
 // each mask point m
 void pr0njmx (int n, int x, 
-              const RVector<int> cumss, 
-              int nc,  int jj, int kk, int mm, int cc0, int binomN,  
-              const RVector<int> PIA0, 
-              const RVector<double> gk0, 
-              const RMatrix<double> Tsk, 
-              std::vector<double> &pjm) {
+    const RVector<int> cumss, 
+    int nc,  int jj, int kk, int mm, int cc0, int binomN,  
+    const RVector<int> PIA0, 
+    const RVector<double> gk0, 
+    const RMatrix<double> Tsk, 
+    std::vector<double> &pjm) {
     
     int c, i, j, k, m, s, ci, gi, pi, ss;
     double Tski;
     bool hazard;
     hazard = (binomN == -2) || (binomN == 0);
-   
+    
     if (hazard) for (i = 0; i < jj*mm; i++) pjm[i] = 0.0;
     else for (i = 0; i < jj*mm; i++) pjm[i] = 1.0;
     ss = cumss[jj];
     for (j = 0; j < jj; j++) {
-	// consider occasions from focal session (j)
-	for (s = cumss[j]; s < cumss[j+1]; s++) {
-	    for (k = 0; k < kk; k++) {
-		ci =  i4(n, s, k, x, nc, ss, kk);
-		c = PIA0[ci] - 1;
-		if (c >= 0) {    
-		    Tski = Tsk(k,s);
-		    for (m = 0; m < mm; m++) {
-			gi = i3(c, k, m, cc0, kk);
-			pi = m * jj + j;
-			if (hazard) {			   
-			    pjm[pi] += gk0[gi] * Tski;  // gk0 is actually hazard 
-			}
-			else {
-			    pjm[pi] *=  pski(binomN, 0, Tski, gk0[gi]);
-			}
-		    }
-		}
-	    }
-	}
+        // consider occasions from focal session (j)
+        for (s = cumss[j]; s < cumss[j+1]; s++) {
+            for (k = 0; k < kk; k++) {
+                ci =  i4(n, s, k, x, nc, ss, kk);
+                c = PIA0[ci] - 1;
+                if (c >= 0) {    
+                    Tski = Tsk(k,s);
+                    for (m = 0; m < mm; m++) {
+                        gi = i3(c, k, m, cc0, kk);
+                        pi = m * jj + j;
+                        if (hazard) {			   
+                            pjm[pi] += gk0[gi] * Tski;  // gk0 is actually hazard 
+                        }
+                        else {
+                            pjm[pi] *=  pski(binomN, 0, Tski, gk0[gi]);
+                        }
+                    }
+                }
+            }
+        }
     }
     if (hazard) {
-	for (i = 0; i < jj*mm; i++) pjm[i] = exp(-pjm[i]);
+        for (i = 0; i < jj*mm; i++) pjm[i] = exp(-pjm[i]);
     }
 }
 //==============================================================================
 
 // [[Rcpp::export]]
 NumericVector PCH0secrjcpp (int type, int x, int nc, int jj,
-			   const IntegerVector cumss, 
-			    int kk, int mm, int cc0,
-			   const IntegerVector PIA0, 
-			   const NumericVector gk0, 
-			   int binomN, 
-			   const NumericMatrix Tsk) {
+    const IntegerVector cumss, 
+    int kk, int mm, int cc0,
+    const IntegerVector PIA0, 
+    const NumericVector gk0, 
+    int binomN, 
+    const NumericMatrix Tsk) {
     
     const RVector<int> cumssR(cumss); 
     const RVector<int> PIA0R(PIA0); 
@@ -128,11 +132,11 @@ NumericVector PCH0secrjcpp (int type, int x, int nc, int jj,
     int j, m, n;
     std::vector<double> pjmat(jj*mm);
     for (n = 0; n<nc; n++) {
-	// primary-session-specific Pr for this animal
-	pr0njmx(n, x, cumssR, nc, jj, kk, mm, cc0, binomN, PIA0R, gk0R, TskR, pjmat);
-	for (j=0; j<jj; j++) {
-	    for (m=0; m<mm; m++) {
-		pdt[j * nc + n] += pjmat[m * jj + j] / mm;
+        // primary-session-specific Pr for this animal
+        pr0njmx(n, x, cumssR, nc, jj, kk, mm, cc0, binomN, PIA0R, gk0R, TskR, pjmat);
+        for (j=0; j<jj; j++) {
+            for (m=0; m<mm; m++) {
+                pdt[j * nc + n] += pjmat[m * jj + j] / mm;
             }
         }
     }
@@ -159,7 +163,8 @@ struct pch1struct : public Worker {
     const RMatrix<double> Tsk;
     const RVector<double> intervals;
     const RVector<int> moveargsi;
-    int   movemodel;
+    int   movementcode;
+    int   edgecode;
     String usermodel;
     const RMatrix<int> kernel;
     const RMatrix<int> mqarray;
@@ -181,19 +186,21 @@ struct pch1struct : public Worker {
         const NumericMatrix Tsk,
         const NumericVector intervals,
         const IntegerVector moveargsi,
-        int   movemodel,
-	String usermodel,
+        int   movementcode,
+        int   edgecode,
+        String usermodel,
         const IntegerMatrix kernel,
         const IntegerMatrix mqarray,
         double cellsize,        
         NumericVector output)    
         : 
-        x(x), type(type), grain(grain), jj(jj), mm(mm), nc(nc), 
-        cumss(cumss), openval0(openval0), PIA0(PIA0), PIAJ(PIAJ), 
-        gk0(gk0), binomN(binomN), Tsk(Tsk), intervals(intervals), 
-        moveargsi(moveargsi), movemodel(movemodel), usermodel(usermodel),
-	kernel(kernel), mqarray(mqarray), 
-        cellsize(cellsize), output(output) 
+            x(x), type(type), grain(grain), jj(jj), mm(mm), nc(nc), 
+            cumss(cumss), openval0(openval0), PIA0(PIA0), PIAJ(PIAJ), 
+            gk0(gk0), binomN(binomN), Tsk(Tsk), intervals(intervals), 
+            moveargsi(moveargsi), movementcode(movementcode), 
+            edgecode(edgecode), usermodel(usermodel),
+            kernel(kernel), mqarray(mqarray), 
+            cellsize(cellsize), output(output) 
     {
         kn = kernel.nrow();
         cc0 = openval0.nrow();
@@ -210,7 +217,7 @@ struct pch1struct : public Worker {
         // work vectors for session-specific real parameter values
         std::vector<double> phij(jj);      // each primary session
         std::vector<double> beta(jj);      // not used if type==1
-        std::vector<double> pm(mm);
+        std::vector<double> alpha(mm);
         std::vector<double> pjmat(jj * mm);
         std::vector<double> moveargs(jj*2);
         std::vector<double> kernelp(kn * (jj-1));
@@ -219,15 +226,16 @@ struct pch1struct : public Worker {
         getbeta (type, n, x, nc, jj, openval0, PIAJ, intervals, phij, beta);
         // primary-session-specific Pr for this animal
         pr0njmx(n, x, cumss, nc, jj, kk, mm, cc0, binomN, PIA0, gk0, Tsk, pjmat);
-        if (movemodel>1) {
+        if (movementcode>1) {
             getmoveargs (n, x, nc, jj, openval0, PIAJ, moveargsi, moveargs);
             if (grain > 0)   // usermodel not allowed with multiple threads 
-		fillkernelparallel (kn, jj, movemodel-2, cellsize, 
-                         kernel, moveargsi, moveargs, kernelp);
-	    else
-		fillkernelp (kn, jj, movemodel-2, cellsize, 
-			     kernel, moveargsi, usermodel, moveargs, kernelp);
+                fillkernelparallel (kn, jj, movementcode-2, cellsize, 
+                    kernel, moveargsi, moveargs, kernelp);
+            else
+                fillkernelp (kn, jj, movementcode-2, cellsize, 
+                    kernel, moveargsi, usermodel, moveargs, kernelp);
         }
+        
         for (b = 1; b <= jj; b++) {
             for (d = b; d <= jj; d++) {
                 pbd = beta[b-1];
@@ -235,18 +243,21 @@ struct pch1struct : public Worker {
                     pbd *= phij[j-1];
                 }
                 pbd *= 1-phij[d-1];
-                // static home ranges: take sum over M of product over J
-                if (movemodel==0) {
-                    for (m=0; m<mm; m++) pm[m] = 1.0/mm;
+                
+                if (movementcode == 0) {
+                    //-------------------------------------------------------------
+                    // static home ranges: take sum over M of product over J
+                    for (m=0; m<mm; m++) alpha[m] = 1.0/mm;
                     for (j = b; j <= d; j++) {
-                        for (m=0; m<mm; m++) pm[m] *= pjmat[m*jj + j - 1];
+                        for (m=0; m<mm; m++) alpha[m] *= pjmat[m*jj + j - 1];
                     }
-                    prw0 = std::accumulate(pm.begin(), pm.end(), 0.0);
+                    prw0 = std::accumulate(alpha.begin(), alpha.end(), 0.0);
                 }
-                else if (movemodel == 1) {
+                else if (movementcode == 1) {
+                    //-------------------------------------------------------------
+                    // uncorrelated home ranges: take product over J of sum over M
                     // over primary sessions in which may have been alive 
                     // centers allowed to differ between primary sessions 
-                    // uncorrelated home ranges: take product over J of sum over M
                     prw0 = 1.0;
                     for (j = b; j <= d; j++) {
                         sumpj = 0;
@@ -256,20 +267,22 @@ struct pch1struct : public Worker {
                         prw0 *= sumpj / mm;   
                     }
                 }
-                else {  // movemodel>1
-                    for (m=0; m<mm; m++) pm[m] = pjmat[m*jj + b - 1] / mm;
+                else {  // movementcode>1
+                    //-------------------------------------------------------------
+                    // moving home ranges
+                    for (m=0; m<mm; m++) alpha[m] = pjmat[m*jj + b - 1] / mm;
                     for (j = b+1; j <= d; j++) {
-                        convolvemq(mm, kn, j-1, mqarray, kernelp, pm);
-                        for (m=0; m<mm; m++) pm[m] *= pjmat[m*jj + j - 1];
+                        convolvemq(mm, kn, j-1, edgecode, mqarray, kernelp, alpha);
+                        for (m=0; m<mm; m++) alpha[m] *= pjmat[m*jj + j - 1];
                     }
-                    prw0 = std::accumulate(pm.begin(), pm.end(), 0.0);
+                    prw0 = std::accumulate(alpha.begin(), alpha.end(), 0.0);
                 }
                 pdt += pbd * (1 - prw0);
             }
         }
-	return pdt;
+        return pdt;
     }
-
+    
     // function call operator that work for the specified range (begin/end)
     void operator()(std::size_t begin, std::size_t end) {
         
@@ -281,44 +294,45 @@ struct pch1struct : public Worker {
 
 // [[Rcpp::export]]
 NumericVector PCH1secrparallelcpp (int x, int type, int grain,
-				   bool individual,
-                                   int jj,  int mm, int nc,
-                                   const IntegerVector cumss, 
-                                   const NumericMatrix openval0, 
-                                   const IntegerVector PIA0, 
-                                   const IntegerVector PIAJ, 
-                                   const NumericVector gk0, 
-                                   int   binomN, 
-                                   const NumericMatrix Tsk, 
-                                   const NumericVector intervals,
-                                   const IntegerVector moveargsi,
-                                   int   movemodel,
-				   const String usermodel,
-                                   const IntegerMatrix kernel,
-                                   const IntegerMatrix mqarray,
-                                   double cellsize) {
+    bool individual,
+    int jj,  int mm, int nc,
+    const IntegerVector cumss, 
+    const NumericMatrix openval0, 
+    const IntegerVector PIA0, 
+    const IntegerVector PIAJ, 
+    const NumericVector gk0, 
+    int   binomN, 
+    const NumericMatrix Tsk, 
+    const NumericVector intervals,
+    const IntegerVector moveargsi,
+    int   movementcode,
+    int   edgecode,
+    const String usermodel,
+    const IntegerMatrix kernel,
+    const IntegerMatrix mqarray,
+    double cellsize) {
     
     NumericVector output(nc); 
     
     // Construct and initialise
     pch1struct pch1 (x, type, grain, jj, mm, nc, 
-                     cumss, openval0, PIA0, PIAJ, gk0, binomN, Tsk, intervals,
-                     moveargsi, movemodel, usermodel, kernel, mqarray, 
-                     cellsize, output);
+        cumss, openval0, PIA0, PIAJ, gk0, binomN, Tsk, intervals,
+        moveargsi, movementcode, edgecode, usermodel, kernel, mqarray, 
+        cellsize, output);
     
     if (individual) {
-	if (grain>0) {
-    	    // Run operator() on multiple threads
-	    parallelFor(0, nc, pch1, grain);
-	}
-	else {
-	    pch1.operator()(0,nc);    // for debugging avoid multithreading to allow R calls
-	}
+        if (grain>0) {
+            // Run operator() on multiple threads
+            parallelFor(0, nc, pch1, grain);
+        }
+        else {
+            pch1.operator()(0,nc);    // for debugging avoid multithreading to allow R calls
+        }
     }
     else {
         // all the same: do once, then copy
-	pch1.operator()(0,1);
-	for (int i = 1; i < nc; i++) output[i] = output[0];
+        pch1.operator()(0,1);
+        for (int i = 1; i < nc; i++) output[i] = output[0];
     }
     
     // Return consolidated result
